#############
# $Id: CheckStats.pm 12454 2011-01-26 10:59:47Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-09-22
# Last Changed On: $Date: 2011-01-26 10:59:47 +0000 (Wed, 26 Jan 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.1/lib/npg_pipeline/analysis/CheckStats.pm $

package npg_pipeline::analysis::CheckStats;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12454 $ =~ /(\d+)/mxs; $r; };
use File::Spec::Functions;
use npg_common::VersionComparison;

extends qw{npg_pipeline::base};
with qw{npg_common::roles::run::path_info};
with qw{npg_common::roles::run::intensities::config};
with qw{npg_common::roles::run::long_info};

Readonly::Scalar our $NEW_RTA_VERSION_GREATER_THAN_CURRENT_RUN_VERSION => -1;

=head1 NAME

npg_pipeline::analysis::CheckStats

=head1 VERSION

$LastChangedRevision: 12454 $

=head1 SYNOPSIS

  use npg_pipeline::analysis::CheckStats;
  my $oStats = npg_pipeline::analysis::CheckStats->new();

=head1 DESCRIPTION

This module checks for and fixes any missing or broken .stats files that are expected for this run.
Ideally this should be run early in the pipeline, so that any processes that expect to use these are not run
until they are fixed.

=head1 SUBROUTINES/METHODS

=head2 check_and_fix_stats

run the check (and fixing) for all stats files

if the flag croak_if_broken_cif_dif is set, then it will not fix the stats files

=cut

sub check_and_fix_stats {
  my ( $self, $arg_refs ) = @_;

  $self->_check_stat_files();

  return ();
}

##############
# private methods

# check .stats files (and fix if no told to croak)
sub _check_stat_files {
  my ( $self ) = @_;

  my $lane_tile_clustercount = $self->lane_tile_clustercount();

  my $intensity_path = $self->intensity_path();
  my $basecalls_path = $intensity_path . q{/BaseCalls}; # we are starting to pass through a different pathway, and until finalised, this needs hard building here

  my $err_count = 0;
  for my $cycle ( $self->first_cycle() .. $self->last_cycle() ){

    while ( my ( $lane , $tile_clustercount_href ) = each %{ $lane_tile_clustercount } ) {

      for my $tile ( keys %{ $tile_clustercount_href } ) {
        my $files = {
          dif    => catfile( $intensity_path, sprintf( 'L%03d', $lane ), qq{C$cycle.1}, sprintf q{s_%s_%d.dif},   $lane, $tile ),
          filter => catfile( $basecalls_path, sprintf q{s_%s_%04d.filter}, $lane, $tile ),
          filter_new_r1 => catfile( $basecalls_path , q{L} . sprintf q{%03d/s_%s_1_%04d.filter}, $lane, $lane, $tile ),
          filter_new_r2 => catfile( $basecalls_path , q{L} . sprintf q{%03d/s_%s_2_%04d.filter}, $lane, $lane, $tile ),
          filter_new_r3 => catfile( $basecalls_path , q{L} . sprintf q{%03d/s_%s_3_%04d.filter}, $lane, $lane, $tile ),
          bcl    => catfile( $basecalls_path, sprintf( 'L%03d', $lane ), qq{C$cycle.1}, sprintf q{s_%s_%d.bcl},   $lane, $tile ),
          stats  => catfile( $basecalls_path, sprintf( 'L%03d', $lane ), qq{C$cycle.1}, sprintf q{s_%s_%d.stats}, $lane, $tile ),
        };

        if ( compare_version( $self->illumina_pipeline_conf()->{old_rta_version_below}, $self->rta_version() ) == $NEW_RTA_VERSION_GREATER_THAN_CURRENT_RUN_VERSION ) {
          delete $files->{filter_new_r1};
          delete $files->{filter_new_r2};
          delete $files->{filter_new_r3};
        } else {
          delete $files->{filter};
          if ( ! $self->is_indexed() || ( $self->is_indexed() && ! $self->is_paired_read() ) ) {
            delete $files->{filter_new_r3};
          }
          if ( ! $self->is_paired_read() && ! $self->is_indexed() ) {
            delete $files->{filter_new_r2};
          }
        }

        eval {

          my $all_seen = 1;
          my $first_failed_to_be_found = q{};
          foreach my $file_type ( sort keys %{ $files } ) {
            if ( ! -e $files->{ $file_type } ) {
              $all_seen = 0;
              if ( ! $first_failed_to_be_found ) {
                $first_failed_to_be_found = $file_type;
              }
              $self->log( qq{Can't find file $files->{ $file_type }});
            }

          }

          if ( $first_failed_to_be_found && $first_failed_to_be_found ne q{stats} ) {
            croak qq{First file to be found missing is $first_failed_to_be_found. Currently no way to fix this};
          }

          if ( ! $all_seen && $self->fix_broken_files() ) {
            $self->_fix_stats_file( $files );
          } elsif ( ! $all_seen ) {
            croak q{missing expected files};
          }

          1;
        } or do {
          croak $EVAL_ERROR;
        };
      }
    }
  }

  return 1;
}

# submits the job to fix the stats file
sub _fix_stats_file {
  my ( $self, $files ) = @_;

  my $cmd = $self->_make_stats_script();
  if ( $self->verbose() ) {
    $cmd .= q{ -verbose};
  }
  $cmd .= q{ } . $files->{dif};
  $cmd .= q{ } . $files->{filter};
  $cmd .= q{ } . $files->{bcl};
  $cmd .= q{ } . $files->{stats};

  $self->log( qq{Fixing stats file command\n\t$cmd} );

  my $output = qx{$cmd};
  $self->log( $output );
  if ( $CHILD_ERROR ) {
    croak q{Unable to successfully run makeStats.pl};
  }

  return 1;
}

# returns the whole path for running the make_stats script
has q{_make_stats_script} => (
  isa => q{Str},
  is  => q{ro},
  lazy_build => 1,
);

sub _build__make_stats_script {
  my ( $self ) = @_;
  return catfile( $self->filesystem_locations_conf()->{src}, $self->pb_cal_pipeline_conf()->{bin}, $self->pb_cal_pipeline_conf()->{make_stats} );
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item File::Spec::Functions

=item npg_common::roles::run::path_info

=item npg_common::roles::run::intensities::config

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
